/*******************************************************************************
 * Filename    : encryption.c
 * Description : provide function to en/decrypt text file and string
 * Created on  : 08/06/08
 *
 * (C) Copyright Promise Technology Inc., 2008
 * All Rights Reserved
 ******************************************************************************/

/* --- standard C lib header files ----------------------------------------- */
#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <stdlib.h>

/* --- internal header files ----------------------------------------------- */
#include "user.h"
#include "encryption.h"

//crypt lib fuctions
extern void encrypt(char *, int);
extern void setkey(char *);

/* --- constant definitions ------------------------------------------------ */
#define ENCODE 0
#define DECODE 1

/* --- private data/data structure section --------------------------------- */

/* --- private routine section ------------------------------------------------ */
static void processEightByteFile(char * srcStr, int codec)
{
    char tmp[64]; int i; 
    
    for ( i = 0; i< 64; i++)
    {
        tmp[i] = ((*(srcStr+i/8))>>(7-(i-i/8*8)))&1;
    }
    encrypt(tmp, codec);
    memset(srcStr, 0, 8);
    for ( i = 0; i< 64; i++)
    {
        *(srcStr+i/8) |= tmp[i]<<(7-(i-i/8*8));
    }            
}

static void processEightByteString(char * srcStr, int begin, int codec)
{
    char tmp[64]; int i; 

    for ( i = 0; i< 64; i++)
    {
        tmp[i] = ((*(srcStr+begin+i/8))>>(7-(i-i/8*8)))&1;
    }
    encrypt(tmp, codec);
    memset(srcStr+begin, 0, 8);
    for ( i = 0; i< 64; i++)
    {
        *(srcStr+begin+i/8) |= tmp[i]<<(7-(i-i/8*8));
    }
   
}


static void processString(char * srcStr, int len, int codec)
{
    int j = 0; int begin = 0; int i;
    for ( i = 0; i < len; i ++)
    {
        j++;
        if ( j == 8)
        {
            processEightByteString(srcStr, begin, codec);
            j = 0;
            begin = i+1;
        }
    }
}
/* --- public routine section ---------------------------------------------- */
/***************************************************************************
* Function Name: encryptTextFile
* Description:  encrypt a text file
* Parameters: [in] pSrcFile, file to be encrypted
*             [in] pDesFile, generated encrypted file
*             [in] pKey, string of 7 byte encryption key
* Return:  success/fail
* Remarks: none.
****************************************************************************/
u32 encryptTextFile(char * pSrcFile, char * pDesFile, char * pKey)
{
    u32 u32Ret = PIERR_NO_ERROR;
    char key[64];
    int i;
    FILE * fp;
    FILE * fp2;
    int count;
    char srcStr[8];
    char realKey[8];
        
    //convert key
    strncpy(realKey, pKey, 8);
    
    for ( i = 0; i< 64; i++)
    {
        key[i] = (realKey[i/8]>>(7-(i-i/8*8)))&1;
    }
    setkey(key);

    fp = fopen(pSrcFile, "r");
    if ( fp == NULL)
        return PIERR_FILE_NOT_FOUND;
    fp2 =  fopen(pDesFile, "wb");
    if ( fp2 == NULL)
        return PIERR_FILE_NOT_FOUND;
        
    count = 0;
    while (( srcStr[count] = fgetc(fp))!= (char)EOF )
    {
       count++;
       if ( count == 8)
       {
           processEightByteFile(srcStr, ENCODE);
           if ( 1 != fwrite(srcStr, 8, 1, fp2))
           {
               u32Ret = PIERR_OUT_OF_MEMORY;
               break;    
           }
           count = 0;
       }
    }
    if ( count != 0)
    {
        srcStr[count] = EOF;
        processEightByteFile(srcStr, ENCODE);
        if ( 1 != fwrite(srcStr, 8, 1, fp2))
        {
             u32Ret = PIERR_OUT_OF_MEMORY;
        }
    }

    fclose(fp);
    fclose(fp2);
    sync();
    return u32Ret; 
}
/***************************************************************************
* Function Name: decryptTextFile
* Description:  decrypt to generate a  text file
* Parameters: [in] pSrcFile, file to be decrypted
*             [in] pDesFile, generated decrypted file
*             [in] pKey, string of 7 byte encryption key
* Return:  success/fail
* Remarks: none.
****************************************************************************/
u32 decryptTextFile(char * pSrcFile, char * pDesFile, char * pKey)
{
    u32 u32Ret = PIERR_NO_ERROR;
    char key[64];
    int i;
    FILE * fp2;
    FILE * fp3;
    int count;
    char srcStr[8];
    char realKey[8];
        
    //convert key
    strncpy(realKey, pKey, 8);
    for ( i = 0; i< 64; i++)
    {
        key[i] = (realKey[i/8]>>(7-(i-i/8*8)))&1;
    }
    setkey(key);

    fp2 =  fopen(pSrcFile, "rb");
    if ( fp2 == NULL)
        return PIERR_FILE_NOT_FOUND;
    fp3 = fopen(pDesFile, "w");
    if ( fp3 == NULL)
        return PIERR_FILE_NOT_FOUND;
        
    count = 0;
    while (fread(srcStr, 8, 1, fp2) != 0)
    {
        processEightByteFile(srcStr, DECODE);
        for ( i=0; i< 8; i++)
        {
            if ( srcStr[i] == (char)EOF)
                break;
            if ( EOF == fputc(srcStr[i], fp3))
            {
                 u32Ret = PIERR_OUT_OF_MEMORY;
                 break; 
            }
        }
    }
    fclose(fp2);
    fclose(fp3);
    sync();
    return u32Ret;   
}
/***************************************************************************
* Function Name: encryptString
* Description:  encrypt a string, the encrypted string is convert to another string
* Parameters: [in] pSrcStr, string to be encrypted
*             [out] pDesStr, generated encrypted string
*             [in] desStrSize, size of the buffer to hold encrypted string
*                               if ( strlen(pSrcStr)+1) is 8 byte aligned
*                                   desStrSize > 3*( (strlen(pSrcStr)+1)
*                               else
*                                   desStrSize > 3*(((strlen(pSrcStr)+1)/8+1)*8)
*             [in] pKey, string of 7 byte encryption key
* Return:  success/fail
* Remarks: none.
****************************************************************************/
u32 encryptString(const unsigned char * pSrcStr, unsigned char * pDesStr, int desStrSize,
        unsigned char * pKey)
{
    u32 u32Ret = PIERR_NO_ERROR;
    char key[64];
    int i;
    unsigned char tmp[8];
    char realKey[8];
    int align = 1;
    int len = strlen(pSrcStr)+1;
    int tmpLen = len;
    unsigned char * tmpString; 
    
    if ( (len - len/8*8) != 0)
    {
        tmpLen = (len/8+1)*8;
        strcpy(tmp, pSrcStr+len/8*8);
        align = 0;
    }
       
    if ( (desStrSize-1) < 3 * tmpLen)
        return PIERR_BUFFER_TOO_SMALL;
        
    tmpString = (unsigned char *)malloc(strlen(pSrcStr)+1);    
    if (tmpString == NULL)
        return PIERR_OUT_OF_MEMORY;
            
    strcpy(tmpString, pSrcStr);    
    //convert key
    strncpy(realKey, pKey, 8);
    for ( i = 0; i< 64; i++)
    {
        key[i] = (realKey[i/8]>>(7-(i-i/8*8)))&1;
    }
    setkey(key);
    
    tmpLen = len/8*8;
    processString(tmpString, tmpLen, ENCODE);
    for ( i=0;i<tmpLen;i++)
    {
        sprintf(pDesStr+i*3,"%03d",tmpString[i]);
    }
    if ( align)
    {
        pDesStr[3*tmpLen] = '\0';
    }
    else
    {
         processString(tmp, 8, ENCODE);
         for ( i=0;i<8;i++)
         {
             sprintf(pDesStr+(tmpLen+i)*3,"%03d",tmp[i]);
         }
         pDesStr[3*(tmpLen+8)] = '\0';  
    }

    free(tmpString);
    
    return u32Ret;   
}
/***************************************************************************
* Function Name: decryptString
* Description:  decrypt to generate original string
* Parameters: [in] pSrcStr, string to be decrypted
*             [out] pDesStr, generated original string
*             [in] desStrSize, size of the buffer to hold decrypted string
*                               if ( strlen(originalString)+1) is 8 byte aligned
*                                   desStrSize >= strlen(originalString)+1
*                               else
*                                   desStrSize >= ((strlen(originalString)+1)/8+1)*8
*             [in] pKey, string of 7 byte encryption key
* Return:  success/fail
* Remarks: none.
****************************************************************************/
u32 decryptString(const unsigned char * pSrcStr, unsigned char * pDesStr, int desStrSize,
        unsigned char * pKey)
{
    u32 u32Ret = PIERR_NO_ERROR;
    char key[64];
    int i;
    char mid[4];
    char realKey[8];
    int len = strlen(pSrcStr);
    unsigned char * tmpString;
        
    if ( desStrSize*3 < strlen(pSrcStr))
        return PIERR_BUFFER_TOO_SMALL;
        
    //convert key
    strncpy(realKey, pKey, 8);
    for ( i = 0; i< 64; i++)
    {
        key[i] = (realKey[i/8]>>(7-(i-i/8*8)))&1;
    }
    setkey(key);

    memset(pDesStr,0, desStrSize);
    mid[3] = '\0';

    tmpString = (unsigned char *)malloc(len+1);
    if (tmpString == NULL)
        return PIERR_OUT_OF_MEMORY;
        
    strcpy(tmpString, pSrcStr);
    
    for ( i=0;i<len/3;i++)
    {
        memcpy(mid, tmpString+i*3, 3);
        pDesStr[i] = atoi(mid);
    }
    //decode
    processString(pDesStr, len/3, DECODE);
    free(tmpString);
    
    return u32Ret; 
}
